@@ -13,8 +13,13 @@ use crate::{test_helpers::EmptyMsg, Contract, ContractWrapper};
 use schemars::JsonSchema;
 use std::fmt::Debug;
 
+use cw0::{parse_execute_response_data, parse_instantiate_response_data};
 use derivative::Derivative;
 
+// Choosing a reply id less than ECHO_EXECUTE_BASE_ID indicates an Instantiate message reply by convention.
+// An Execute message reply otherwise.
+pub(crate) const EXECUTE_REPLY_BASE_ID: u64 = i64::MAX as u64;
+
 #[derive(Debug, Clone, Serialize, Deserialize, Derivative)]
 #[derivative(Default(bound = "", new = "true"))]
 pub struct Message<ExecC>
@@ -27,17 +32,35 @@ where
     pub events: Vec<Event>,
 }
 
+// This can take some data... but happy to accept {}
+#[derive(Debug, Clone, Serialize, Deserialize, Derivative)]
+#[derivative(Default(bound = "", new = "true"))]
+pub struct InitMessage<ExecC>
+where
+    ExecC: Debug + PartialEq + Clone + JsonSchema + 'static,
+{
+    pub data: Option<String>,
+    pub sub_msg: Option<Vec<SubMsg<ExecC>>>,
+}
+
 #[allow(clippy::unnecessary_wraps)]
 fn instantiate<ExecC>(
     _deps: DepsMut,
     _env: Env,
     _info: MessageInfo,
-    _msg: EmptyMsg,
+    msg: InitMessage<ExecC>,
 ) -> Result<Response<ExecC>, StdError>
 where
     ExecC: Debug + PartialEq + Clone + JsonSchema + 'static,
 {
-    Ok(Response::default())
+    let mut res = Response::new();
+    if let Some(data) = msg.data {
+        res = res.set_data(data.into_bytes());
+    }
+    if let Some(msgs) = msg.sub_msg {
+        res = res.add_submessages(msgs);
+    }
+    Ok(res)
 }
 
 #[allow(clippy::unnecessary_wraps)]
@@ -71,17 +94,35 @@ fn reply<ExecC>(_deps: DepsMut, _env: Env, msg: Reply) -> Result<Response<ExecC>
 where
     ExecC: Debug + PartialEq + Clone + JsonSchema + 'static,
 {
+    let res = Response::new();
     if let Reply {
+        id,
         result:
             ContractResult::Ok(SubMsgExecutionResponse {
                 data: Some(data), ..
             }),
-        ..
     } = msg
     {
-        Ok(Response::new().set_data(data))
+        // We parse out the WasmMsg::Execute wrapper...
+        // TODO: Handle all of Execute, Instantiate, and BankMsg replies differently.
+        let parsed_data;
+        if id < EXECUTE_REPLY_BASE_ID {
+            parsed_data = parse_instantiate_response_data(data.as_slice())
+                .map_err(|e| StdError::generic_err(e.to_string()))?
+                .data;
+        } else {
+            parsed_data = parse_execute_response_data(data.as_slice())
+                .map_err(|e| StdError::generic_err(e.to_string()))?
+                .data;
+        }
+
+        if let Some(data) = parsed_data {
+            Ok(res.set_data(data))
+        } else {
+            Ok(res)
+        }
     } else {
-        Ok(Response::new())
+        Ok(res)
     }
 }
 
